from ftw.tabbedview.browser import listing


class Tab(listing.CatalogListingView):
    """Base view class for workspace tabs.

    See ftw.tabbedview.browser.views.BaseListingView for attributes and
    methods that are meant to be overwritten to change the behavior.
    """
